/**********************************************************************************************
    Copyright (C) 2007 Oliver Eichler oliver.eichler@gmx.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA

  Garmin and MapSource are registered trademarks or trademarks of Garmin Ltd.
  or one of its subsidiaries.

**********************************************************************************************/

#include "IDeviceDefault.h"

#include <iostream>

using namespace Garmin;
using namespace std;

IDeviceDefault::IDeviceDefault()
{

}

IDeviceDefault::~IDeviceDefault()
{

}

void IDeviceDefault::setPort(const char * p)
{
    port = p;
}

void IDeviceDefault::uploadMap(const uint8_t * mapdata, uint32_t size, const char * key, void (*callback)(uint32_t,uint32_t,void*), void* data)
{
    try{
        _acquire();
        _uploadMap(mapdata, size, key, callback, data);
        _release();
    }
    catch(exce_t& e){
        _release();
        lasterror = "Failed to upload maps. " + e.msg;
        throw (int)e.err;
    }
}

void IDeviceDefault::queryMap(std::list<Map_t>& maps)
{
    try{
        _acquire();
        _queryMap(maps);
        _release();
    }
    catch(exce_t& e){
        _release();
        lasterror = "Failed to query loaded maps. " + e.msg;
        throw (int)e.err;
    }
}

void IDeviceDefault::downloadWaypoints(std::list<Garmin::Wpt_t>& waypoints)
{
    try{
        _acquire();
        _downloadWaypoints(waypoints);
        _release();
    }
    catch(exce_t& e){
        _release();
        lasterror = "Failed to download waypoints. " + e.msg;
        throw (int)e.err;
    }
}

void IDeviceDefault::uploadWaypoints(std::list<Garmin::Wpt_t>& waypoints)
{
    try{
        _acquire();
        _uploadWaypoints(waypoints);
        _release();
    }
    catch(exce_t& e){
        _release();
        lasterror = "Failed to upload waypoints. " + e.msg;
        throw (int)e.err;
    }
}

void IDeviceDefault::downloadTracks(std::list<Garmin::Track_t>& tracks)
{
    try{
        _acquire();
        _downloadTracks(tracks);
        _release();
    }
    catch(exce_t& e){
        _release();
        lasterror = "Failed to download tracks. " + e.msg;
        throw (int)e.err;
    }

}

const std::string& IDeviceDefault::getCopyright()
{
    return copyright;
}

const std::string& IDeviceDefault::getLastError()
{
    return lasterror;
}

void IDeviceDefault::_uploadMap(const uint8_t * , uint32_t , const char * , void (*)(uint32_t,uint32_t,void*), void* )
{
    throw exce_t(errNotImpl,"uploadMap(): this method is not implemented for your device.");
}

void IDeviceDefault::_queryMap(std::list<Map_t>& )
{
    throw exce_t(errNotImpl,"queryMap(): this method is not implemented for your device.");
}

void IDeviceDefault::_downloadWaypoints(std::list<Garmin::Wpt_t>& )
{
    throw exce_t(errNotImpl,"downloadWaypoints(): this method is not implemented for your device.");
}

void IDeviceDefault::_uploadWaypoints(std::list<Garmin::Wpt_t>& )
{
    throw exce_t(errNotImpl,"uploadWaypoints(): this method is not implemented for your device.");
}

void IDeviceDefault::_downloadTracks(std::list<Garmin::Track_t>& )
{
    throw exce_t(errNotImpl,"downloadTracks(): this method is not implemented for your device.");
}


